@@ -48,9 +48,9 @@ G_DEFINE_TYPE_WITH_CODE (GClueModemGPS,
                          GCLUE_TYPE_LOCATION_SOURCE,
                          G_ADD_PRIVATE (GClueModemGPS))
 
-static GClueLocationSourceStartResult
+static gboolean
 gclue_modem_gps_start (GClueLocationSource *source);
-static GClueLocationSourceStopResult
+static gboolean
 gclue_modem_gps_stop (GClueLocationSource *source);
 
 static void
@@ -80,15 +80,14 @@ on_gps_enabled (GObject      *source_object,
                 GAsyncResult *result,
                 gpointer      user_data)
 {
-        g_autoptr(GError) error = NULL;
+        GClueModemGPS *source = GCLUE_MODEM_GPS (user_data);
+        GError *error = NULL;
 
-        if (!gclue_modem_enable_gps_finish (GCLUE_MODEM (source_object),
+        if (!gclue_modem_enable_gps_finish (source->priv->modem,
                                             result,
                                             &error)) {
-                if (error && !g_error_matches (error, G_IO_ERROR,
-                                               G_IO_ERROR_CANCELLED)) {
-                        g_warning ("Failed to enable GPS: %s", error->message);
-                }
+                g_warning ("Failed to enable GPS: %s", error->message);
+                g_error_free (error);
         }
 }
 
@@ -156,7 +155,7 @@ gclue_modem_gps_init (GClueModemGPS *source)
         GClueModemGPSPrivate *priv;
         GClueMinUINT *threshold;
 
-        source->priv = gclue_modem_gps_get_instance_private (source);
+        source->priv = G_TYPE_INSTANCE_GET_PRIVATE ((source), GCLUE_TYPE_MODEM_GPS, GClueModemGPSPrivate);
         priv = source->priv;
 
         priv->cancellable = g_cancellable_new ();
@@ -169,10 +168,10 @@ gclue_modem_gps_init (GClueModemGPS *source)
                                           source);
         threshold = gclue_location_source_get_time_threshold
                         (GCLUE_LOCATION_SOURCE (source));
-        g_signal_connect_object (threshold,
-                                 "notify::value",
-                                 G_CALLBACK (on_time_threshold_changed),
-                                 source, 0);
+        g_signal_connect (threshold,
+                          "notify::value",
+                          G_CALLBACK (on_time_threshold_changed),
+                          source);
 }
 
 static void
@@ -210,21 +209,18 @@ gclue_modem_gps_get_singleton (void)
 
 static void
 on_fix_gps (GClueModem *modem,
-            const char *nmeas[],
+            const char *gga,
             gpointer    user_data)
 {
         GClueLocationSource *source = GCLUE_LOCATION_SOURCE (user_data);
-        GClueLocation *prev_location;
-        g_autoptr(GClueLocation) location = NULL;
-        g_autoptr(GError) error = NULL;
+        GClueLocation *location;
+        GError *error = NULL;
 
-        prev_location = gclue_location_source_get_location (source);
-        location = gclue_location_create_from_nmeas (nmeas,
-                                                     prev_location,
-                                                     &error);
+        location = gclue_location_create_from_gga (gga, &error);
 
         if (error != NULL) {
             g_warning ("Error: %s", error->message);
+            g_clear_error (&error);
 
             return;
         }
@@ -233,21 +229,18 @@ on_fix_gps (GClueModem *modem,
                                             location);
 }
 
-static GClueLocationSourceStartResult
+static gboolean
 gclue_modem_gps_start (GClueLocationSource *source)
 {
         GClueLocationSourceClass *base_class;
         GClueModemGPSPrivate *priv;
-        GClueLocationSourceStartResult base_result;
 
-        g_return_val_if_fail (GCLUE_IS_LOCATION_SOURCE (source),
-                              GCLUE_LOCATION_SOURCE_START_RESULT_FAILED);
+        g_return_val_if_fail (GCLUE_IS_LOCATION_SOURCE (source), FALSE);
         priv = GCLUE_MODEM_GPS (source)->priv;
 
         base_class = GCLUE_LOCATION_SOURCE_CLASS (gclue_modem_gps_parent_class);
-        base_result = base_class->start (source);
-        if (base_result != GCLUE_LOCATION_SOURCE_START_RESULT_OK)
-                return base_result;
+        if (!base_class->start (source))
+                return FALSE;
 
         g_signal_connect (priv->modem,
                           "fix-gps",
@@ -260,23 +253,21 @@ gclue_modem_gps_start (GClueLocationSource *source)
                                         on_gps_enabled,
                                         source);
 
-        return base_result;
+        return TRUE;
 }
 
-static GClueLocationSourceStopResult
+static gboolean
 gclue_modem_gps_stop (GClueLocationSource *source)
 {
         GClueModemGPSPrivate *priv = GCLUE_MODEM_GPS (source)->priv;
         GClueLocationSourceClass *base_class;
-        g_autoptr(GError) error = NULL;
-        GClueLocationSourceStopResult base_result;
+        GError *error = NULL;
 
         g_return_val_if_fail (GCLUE_IS_LOCATION_SOURCE (source), FALSE);
 
         base_class = GCLUE_LOCATION_SOURCE_CLASS (gclue_modem_gps_parent_class);
-        base_result = base_class->stop (source);
-        if (base_result == GCLUE_LOCATION_SOURCE_STOP_RESULT_STILL_USED)
-                return base_result;
+        if (!base_class->stop (source))
+                return FALSE;
 
         g_signal_handlers_disconnect_by_func (G_OBJECT (priv->modem),
                                               G_CALLBACK (on_fix_gps),
@@ -288,7 +279,8 @@ gclue_modem_gps_stop (GClueLocationSource *source)
                                               &error)) {
                         g_warning ("Failed to disable GPS: %s",
                                    error->message);
+                        g_error_free (error);
                 }
 
-        return base_result;
+        return TRUE;
 }
